#! /bin/bash
# (C) 2009 Mithat Konar
# 2009-05-31

# GUI interface to set-Xres-DPI.sh
# Requires zenity.

# TODO: for the next version, read the existing DPI value (if any) and show as default value.

# ===========
# UI strings
# ===========
ProgramName="Set DPI"		# name used to refer to this program

uiMainTitle="$ProgramName"
uiMainPrompt=\
"This applet lets you set the DPI (dots-per-inch) used to render screen
fonts to a value that is different from the system default. Note that not
all programs will honor this setting, even though they should.

A typical DPI value is 96; other common values are 120, 100, 85, and 75.

Enter your desired DPI (or 0 to use the system default):"

uiNoInputErrTitle="DPI cannot be blank"
uiNoInputErr="\
You must specify a positive number for the new DPI setting.

Enter the DPI (or 0 to use the system default):"

uiDPIErrTitle="Invalid setting"
uiDPIErr="The DPI must be a single positive number.

Enter the DPI (or 0 to use the system default):"

uiFollowupTitle="Logout/login required"
uiFollowup=\
"Your DPI setting has been changed.

To see the effects of the new setting, you must logout and login again."

uiFileErrTitle="File access error"
uiFileErr="Either the file I am trying to write to does not exist or I \
do not have permission to write to it.
		
$ProgramName will now quit."

uiUnknownErrTitle="Unknown error"
uiUnknownErr="An unknown error has occurred.

$ProgramName will now quit."


# ================
# global constants
# ================

APPDIR=`dirname "$0"`
cd -P "$APPDIR"
APPDIR="`pwd`"
iconPath="./icons/setDPI.png"

theDPI=96

# exit codes from set-Xres-DPI.sh
notEnoughParams=3		# not enough parameters were passed to the script
dpiNotValid=4			# the DPI parameter was not valid (must be non-negative)

# ===========
# functions
# ===========
function currentDPI()
# returns the value of the last valid DPI statement in ~/.Xresources
# via global theDPI
{
	rscFile=~/.Xresources
	if [ -e $rscFile ] ; then
		while read line ; do
			# find line that begin with [<whitepace>]Xft.dpi
			edLine=`echo $line | grep -e ^[[:space:]]*Xft.dpi`
			if [ "$edLine" != "" ] ; then
				# strip away [<whitepace>]Xft.dpi[<whitepace>]:[<whitepace>]
				edLine=`echo $edLine | sed "s/^[[:space:]]*Xft.dpi[[:space:]]*:[[:space:]]*//g"`
				# strip away trailing whitespace
				edLine=`echo $edLine | sed 's/[[:space:]]*$//'`
				# take the last valid DPI value
				theDPI=$edLine
			fi
		done < $rscFile
	fi
}

# ===========
# "main"
# ===========

# check for zenity
isZenity=`which zenity`
if [ -z $isZenity ] ; then
	echo "${0}: $ui_zenityNotFound"
	exit 1
fi

rv=-1	# return value used to control looping
zprompt="$uiMainPrompt"
ztitle="$uiMainTitle"
currentDPI
while [ $rv -ne 0 ] ; do
	requestedDPI=$(zenity --entry --window-icon "$iconPath" --title "$ztitle" --text "$zprompt" --entry-text "$theDPI")
	rv=$?
	if [ $rv -ne 0 ]; then		# if user cancelled, get out of here
		echo "$0: user cancelled" 1>&2
		exit $rv
	fi

	# verify that user entered something and act on it
	if [ -n "$requestedDPI" ]; then		
		"$APPDIR/set-Xres-DPI.sh" $requestedDPI
		rv=$?
	else
		rv=$notEnoughParams
	fi

	# depending on what happened above, do something
	case $rv in 
	0)
		# success! show exit message and be done
		zenity "$iconPath" --info --title "$uiFollowupTitle" --text "$uiFollowup"
		;;
	$notEnoughParams)
		# change prompt message and try again
		ztitle="$uiNoInputErrTitle"
		zprompt="$uiNoInputErr"
		;;
	$dpiNotValid)
		# change prompt message and try again
		ztitle="$uiDPIErrTitle"
		zprompt="$uiDPIErr"
		;;
	*)
		# show generic error messsage and exit
		zenity --error --title "$uiUnknownErrTitle" --text "$uiUnknownErr"
		exit $rv
		;;
	esac
done
exit
