#! /bin/bash
#Mithat Konar - 2009/01/02

# Changes dpi references in ~/.Xresources files to $1.
# Setting DPI in .Xresources will override system DPI setting.
# Examples:
#   To set user to 96 dpi:
#   set-Xres-DPI.sh 96
### The script will make a backup of the original file.
# But you should still USE WITH GREAT CAUTION.

# exit codes
notEnoughParams=3		# not enough parameters were passed to the script
dpiNotValid=4			# the DPI parameter was not valid (must be non-negative)

# check 1st parameter -- it must be a non-negative number
if [ -z "$1" ]; then
	echo "$0: Not enough parameters" 1>&2
	exit $notEnoughParams
else
	# test that it's a number >= 0
	isNonNegNum=$(echo "$1" | sed "s/[0-9]//g;s/\.//")	# strip out allowed chars
	if [ -n "$isNonNegNum" ] ; then
		echo "$0: DPI must be a non-negative number" 1>&2
		exit $dpiNotValid
	fi
fi

# some constants
commentStr="! This file was modified by set-Xres-DPI.sh on"

# set working files
cfgFile=~/.Xresources
suffix=$(date +%s).BAK
if [ -e "$cfgFile" ]; then
	bckpFile="${cfgFile}.${suffix}"
else
	touch "$cfgFile"
fi
tmpFile=/tmp/set-Xres-DPI.$$.tmp
rm -f $tmpFile					# make doubly sure tmpFile is empty

# backup original file
cp -dp "$cfgFile" "$bckpFile"	

# There are two ways we could go about this.
# The easy way:
# delete any lines that begins with "Xft.dpi" or "[whitespace]Xft.dpi".
# then, if $1 != 0, add a "Xft.dpi: $1" line.
# The hard way:
# if $1 == 0,
#   delete any lines that begin with "Xft.dpi" or "[whitespace]Xft.dpi".
#   add "Xft.dpi: $1"
# else
#   delete everything to the right of any lines stating with
#   "Xft.dpi" or "[whitespace]Xft.dpi"
#   then find any lines starting with "Xft.dpi" or "[whitespace]Xft.dpi"
#   and append ": $1"

# We'll do it the easy way:
# first, strip out comments added by this script previously
sed "/$commentStr/ d" "$cfgFile" >> "$tmpFile"
cp "$tmpFile" "$cfgFile" 	# copy temp file into config file
rm -f $tmpFile				# remove temp file

# second, delete any lines that begin with "Xft.dpi" or "[whitespace]Xft.dpi".
sed "/^[ \t]*Xft.dpi/ d" "$cfgFile" >> "$tmpFile"

# third, if $1 is zero, append comment, else append Xft.dpi command and comment.
# we gotta jump thru some hoops to handle 'non-integer" zero. The below strips away
# 'permitted' characters from $1. If anything is left, the result must be non-zero.
isZero=$(echo "$1" | sed "s/^[+-]//;s/[0]//g;s/\.//")
if [ -z "$isZero" ] ; then
	echo -e "$commentStr $(date) to remove the DPI option." >> "$tmpFile"
else
	echo -e "Xft.dpi: $1" >> "$tmpFile"
	echo -e "$commentStr $(date) to $1 DPI." >> "$tmpFile"
fi

# copy tmpFile back to cfgFile, remove tmpFile
cp "$tmpFile" "$cfgFile"
rm -f $tmpFile
