#! /bin/bash
# (C) 2009 Mithat Konar
# 2009-06-23

# GUI interface to ./set-xdm-DPI.sh
# Requires zenity.
# User must be root.

# ===========
# UI strings
# ===========
ProgramName="System DPI"		# name used to refer to this program

ui_zenityNotFound="This package requires zenity to work."

uiMustBeRootTitle="Must be root"
uiMustBeRoot="You must be the root user to set the system DPI."

uiMainTitle="$ProgramName"
uiMainPrompt=\
"This applet configures the system's default DPI (dots-per-inch) setting used
to render fonts. It will only work if you are using XDM as the login manager.

A very common DPI value is 96; other common values are 120, 100, 85, and 75.

(Current`xdpyinfo | grep resolution`)

Enter the DPI (or 0 to use the system default):"

uiNoInputErrTitle="DPI cannot be blank"
uiNoInputErr="\
You must specify a positive number for the new DPI setting.

Enter the DPI (or 0 to use the system default):"

uiDPIErrTitle="Invalid setting"
uiDPIErr="The DPI must be a single positive number.

Enter the DPI (or 0 to use the default):"

uiFollowupTitle="Restart required"
uiFollowup=\
"The system's DPI setting has been changed.

Your new setting will take effect after a system reboot."

uiFileErrTitle="File access error"
uiFileErr="Either the file I am trying to write to does not exist or I \
do not have permission to write to it.
		
$ProgramName will now quit."

uiUnknownErrTitle="Unknown error"
uiUnknownErr="An unknown error has occurred.

$ProgramName will now quit."

# ================
# global constants
# ================

APPDIR=`dirname "$0"`
cd -P "$APPDIR"
APPDIR="`pwd`"

iconPath="./icons/setDPI.png"

# exit codes from ./set-xdm-DPI.sh
notEnoughParams=3		# not enough parameters were passed to the script
dpiNotValid=4			# the DPI parameter was not valid (must be non-negative)
outputFileNotValid=5	# the config file does not exist or you do not have write permission
permissionProb=126		# Command invoked cannot execute

# ===========
# "main"
# ===========

# check that user is root, show error if not
effectiveUser=`whoami`
if [ "$effectiveUser" != "root" ] ; then
	zenity --error --title "$uiMustBeRootTitle" --text "$uiMustBeRoot"
	echo "$0: Must be root." 1>&2
	exit $permissionProb
fi

# check for zenity
isZenity=`which zenity`
if [ -z $isZenity ] ; then
	echo "${0}: $ui_zenityNotFound"
	exit 1
fi

rv=-1	# return value used to control looping
zprompt="$uiMainPrompt"
ztitle="$uiMainTitle"
while [ $rv -ne 0 ] ; do
	requestedDPI=$(zenity --entry --window-icon "$iconPath" --title "$ztitle" --text "$zprompt" --entry-text "96")
	rv=$?
	if [ $rv -ne 0 ]; then		# if user cancelled, get out of here
		echo "$0: user cancelled" 1>&2
		exit $rv
	fi

	# verify that user entered something and act on it
	if [ -n "$requestedDPI" ]; then		
		./set-xdm-DPI.sh $requestedDPI
		rv=$?
	else
		rv=$notEnoughParams
	fi

	# depending on what happened above...
	case $rv in 
	0)
		# success! show exit message and be done
		zenity "$iconPath" --info --title "$uiFollowupTitle" --text "$uiFollowup"
		;;
	$notEnoughParams)
		# change prompt message and try again
		ztitle="$uiNoInputErrTitle"
		zprompt="$uiNoInputErr"
		;;
	$dpiNotValid)
		# change prompt message and try again
		ztitle="$uiDPIErrTitle"
		zprompt="$uiDPIErr"
		;;
	$outputFileNotValid)
		# show error message and exit
		zenity --error --title "$uiFileErrTitle" --text "$uiFileErr"
		exit $rv
		;;
	*)
		# show generic error messsage and exit
		zenity --error --title "$uiUnknownErrTitle" --text "$uiUnknownErr"
		exit $rv
		;;
	esac
done
exit
