#!/bin/bash

#Mithat Konar - 26 May 2008

# GUI wizard wrapper for Debian's chfn (run as root).

# ==========
# UI strings
# ==========
uiXdialogNotFound="Xdialog was not found.
Please install Xdialog to use this script."
uiSedNotFound="sed was not found.
Please install sed to use this script."

uiChangeFingerData="Change finger data"
uiOk="Ok"
uiCancel="Cancel"
uiConfirmDetails="Confirm change"
uiPressKey2Exit="Press ENTER to continue . . ."

uiChangeFingerDataIntro="Select the name of the user whose finger data you want to change:"
uiChangeFingerDataHelp="\
If you don't see the name of the user you want to change in the\n\
list, the user is not a normal user. You will need to use the\n\
command-line \"chfn\" tool instead of this one to make the changes."

uiTheEndMessage="Finished changing %s's finger data."

# =======
# Globals
# =======
APPDIR=`dirname "$0"`
cd -P "$APPDIR"
APPDIR="`pwd`"

usersIcon="${APPDIR}/icons/users.xpm"
warningIcon="${APPDIR}/icons/warning.xpm"
errorIcon="${APPDIR}/icons/error.xpm"
tmpFile=/tmp/changepasswd.tmp.$$

# the following are used for "returning" values from functions
gStepNum=1		# step in the wizard
gUsername=""	# stores new username

# =========
# Functions
# =========

function getUsername()
# getUsername
# prompt for username to change passwd.
# copy username to global gUsername.
# check for name validity and return 0 if name is valid; 1 otherwise
# This is a "wizard function"; it sets the global gStepNum to
# gStepNum+1 (next) or gStepNum-1 (previous).
{
	local tmpFile=/tmp/getUsername.tmp.$$
	local retval=-1
	local nameStr=""
	local tmpResult=""
	
	# make a list of users in $tmpFile
	cat /etc/passwd | grep /home/ | grep -v /bin/false | cut -d":" -f1 > $tmpFile
	sort -n --output=$tmpFile $tmpFile		# sort the list

	local buttonsString=""
	while read line
	do
	{
		buttonsString="$buttonsString $line $line \"off\""
	}
	done< $tmpFile
	rm -f $tmpFile							# we're done with tmpFile

	Xdialog --icon "$usersIcon" \
			--left \
			--wrap \
			--wizard \
			--help "$uiChangeFingerDataHelp" \
			--title "$uiChangeFingerData" \
			--no-tags \
			--radiolist "$uiChangeFingerDataIntro" 0 0 5 $buttonsString 2> $tmpFile
	retval=$?	
	
	nameStr=`sed -n '1p' "$tmpFile"`		# get the filename in $tmpFile
	rm -f $tmpFile							# we're done with tmpFile
	
	# depending on user response, exit or set to previous step and return
	if [ $retval -eq 1 ]; then		# cancel
		exit 1
	elif [ $retval -eq 3 ]; then	# previous
		let "gStepNum=$gStepNum-1"
		return
	fi

	# set globals used for function output
	gUsername="$nameStr"						# set new user name
		
	let "gStepNum=$gStepNum+1"
	return 0
}

function changeFinger()
# changeFinger $1
# $1 is <username>, a valid name is assumed!
# Asks to confirm actions
# This is a "wizard function"; it sets the global gStepNum to
# gStepNum+1 (next) or gStepNum-1 (previous).
{
	local u_name=""
	local cmdStr1=""
	local confirmStr=""
	local retval=""
	
	# get/check input params
	if [ -n "$1" ]; then
		u_name="$1"
	else
		echo "ERROR: not enough parameters" >&2
		exit 1
	fi

	# assemble command strings
	cmdStr1="chfn $u_name"		# start building command to execute

	x-terminal-emulator -ls -title "$uiChangeFingerData" -n "$uiChangeFingerData"\
		-e /bin/sh -c "$cmdStr1 ;\
		               echo ;\
		               read -p \"${uiPressKey2Exit}\""

	# move to next step
	let "gStepNum=$gStepNum+1"
}

function theEnd
# Lets user know they have finished the process
{
	Xdialog --icon "$usersIcon" \
			--left \
			--wrap \
			--title "$uiChangeFingerData" \
			--msgbox "`printf \"$uiTheEndMessage\" \"$gUsername\"`" 0 0 
}

# ======
# "Main"
# ======

# Test for Xdialog
isExist=`which Xdialog`
if [ -z $isExist ] ; then
	echo "${0}: $uiXdialogNotFound"
	xmessage -center -buttons Ok "$uiXdialogNotFound"
	exit 1
fi
# Test for sed
isExist=`which sed`
if [ -z $isExist ] ; then
	#echo "${0}: $uiSedNotFound"
	xmessage -center -buttons Ok "$uiSedNotFound"
	exit 1
fi

# go!
while true ; do
	case $gStepNum in
		0)
			# signal to calling ./ManageUsers.sh that it needs to show GUI
			# with "special" return value of 3
			exit 3
			;;
		1)
			getUsername
			continue
			;;
		2)
			changeFinger "$gUsername"
			continue;
			;;
		3)
			theEnd
			break
			;;
		*)
			echo "ERROR: undefined program state" >&2
			exit 1
	esac
done
