#!/bin/bash

# Launchpad for various user management scripts used by root
#
# 17 Jan 2009
# 
# All files (C) 2009 Mithat Konar

APPDIR=`dirname "$0"`
cd -P "$APPDIR"
APPDIR="`pwd`"
usersIcon="${APPDIR}/icons/users.xpm"
errorIcon="${APPDIR}/icons/error.xpm"
tmpFile=/tmp/manageusers.tmp.$$

# ==========
# UI strings
# ==========

uiXdialogNotFound="Xdialog was not found.
Please install Xdialog to use this script."

uiMustBeRootTitle="Must be root"
uiMustBeRoot="You must be the root user to manage users."

uiManageUsers="Manage users"
uiManageUsersText="What do you want to do?"

uiMakeUser="Create a new user"
uiDeleteUser="Delete a user"
uiChangePasswd="Change a user's password"
uiChangeFinger="Change a user's finger data"
uiSkinnyReset="Reset a user's environment"

# =========
# constants
# =========

tagMakeUser="makeUser"
tagDeleteUser="deleteUser"
tagChangePasswd="changePasswd"
tagChangeFinger="changeFinger"
tagSkinnyReset="skinnyReset"

# =========
# functions
# =========

function intro()
# Show a GUI that asks the user what they want the wizard to do.
# Return the resired action in the global 'doThis', where 'doThis' is a
# global tag, e.g: tagMakeUser, tagDeleteUser, tagChangePasswd, tagChangeFinger.
{
	local retval=0
	
	Xdialog --icon "$usersIcon" \
			--left \
			--wrap \
			--title "$uiManageUsers" \
			--no-tags \
			--radiolist "$uiManageUsersText" 0 0 6 \
	"$tagMakeUser" "$uiMakeUser" on \
	"$tagDeleteUser" "$uiDeleteUser" off \
	"$tagChangePasswd" "$uiChangePasswd" off \
	"$tagChangeFinger" "$uiChangeFinger" off \
	"$tagSkinnyReset" "$uiSkinnyReset" off \
	2> $tmpFile
	retval=$?	

	doThis=`sed -n '1p' "$tmpFile"`		# get the result from $tmpFile
	rm -f $tmpFile						# we're done with tmpFile
		
	if [ $retval -eq 1 ]; then		# cancel
		doThis=1
		return 1
	fi
}

# ======
# "Main"
# ======

# Test for Xdialog
isXdialog=`which Xdialog`
if [ -z $isXdialog ] ; then
	echo "$uiXdialogNotFound"
	xmessage -center -buttons Ok "$uiXdialogNotFound"
	exit 1
fi

effectiveUser=`whoami`
if [ "$effectiveUser" != "root" ] ; then
	Xdialog --icon "$errorIcon" \
			--left \
			--wrap \
			--title "$uiMustBeRootTitle" \
			--msgbox "$uiMustBeRoot" 0 0
	exit 1
fi

# The value 3 is a special value used to indicate that the wizard is starting
# from the beginning. Scripts invoked by this wizard should return 3 when
# their "previous" key is pressed while showing their first screen.
rv=3
while [ $rv -eq 3 ] ; do
	intro
	rv=$?
	case $doThis in
		$tagMakeUser)
			./newuser.sh
			rv=$?
			;;
		$tagDeleteUser)
			./removeuser.sh
			rv=$?
			;;
		$tagChangePasswd)
			./changepasswd.sh
			rv=$?
			;;
		$tagChangeFinger)
			./changefinger.sh
			rv=$?
			;;
		$tagSkinnyReset)
			./reset-skinny.sh
			rv=$?
			;;
		1)
			echo "User cancelled" >&2
			break
			;;
		*)
			echo "ERROR: undefined program state" >&2
			exit 1
			;;
	esac
done
exit $rv
