#!/bin/bash

#Mithat Konar - 2009-08-04

# GUI wizard wrapper for Debian's adduser. Optionally adds user to
# "additional groups" and/or plugdev and/or makes user a superuser.

# ==========
# UI strings
# ==========
uiXdialogNotFound="Xdialog was not found.
Please install Xdialog to use this script."

uiAddUser="Add new user"
uiOk="Ok"
uiCancel="Cancel"
uiConfirmDetails="Confirm user details"
uiPressKey2Exit="Press ENTER to continue . . ."

uiNameRules=\
"Usernames may contain only lower-case letters, numbers, or\n\
the underscore character. They must start with a lower-case letter.\n\
\n\
Spaces, dashes, upper-case letters, etc. are not permitted."

uiEnterNewName="Enter new username:"
uiNoBlankNames="The username cannot be blank."
uiInvalidName="\"%s\" is not a valid username."
uiUserAlreadyExists="The user \"%s\" already exists on this system."

uiSetOptionsInfo=\
"Set the options below to determine what the new user can do."
uiSetOptionsHelp=\
"[ ] Allow user to change audio settings ...\n\
     Adds the new user to the 'extra groups' listed in the file \"adduser.conf\"\n\
     using adduser's --add_extra_groups flag. For more information consult\n\
     the man pages for the 'adduser' command. Also adds the user to the \n\
     'netdev' group.\n\
\n\
[ ] Allow user to automatically mount ...
     Adds the user to the 'plugdev' group, which enables automounting\n\
     of removable media for this user. For more information, consult\n\
     the man pages for 'pmount' and 'ivman'.\n\
\n\
[ ] Make user an administrator
     Adds the user to the list of superusers. Superusers are allowed to\n\
     add and remove software and make system-wide configuration changes.\n\
     If you elect to make the user an administrator, the user will be made\n\
     a member of the 'wheel' group, and it is assumed that the 'wheel'\n\
     group has been given superuser privileges in the \"/etc/sudoers\" file."

uiSetOptionsExtraDesc="Allow user to change audio settings, write to optical media, and do similar tasks."
uiSetOptionsPlugdevDesc="Allow user to automatically mount and unmount removable media."
uiSetOptionsSuperUserDesc="Make user an administrator."

uiConfirmUsername="The next step will create a new user called \"%s\" who is"
uiExtraGroupsYes="able to change audio settings, write to optical media, and do similar tasks"
uiExtraGroupsNo="un$uiExtraGroupsYes"
uiPlugdevYes="able to mount/unmount removable media"
uiPlugdevNo="un$uiPlugdevYes"
uiSuperUserYes="an administrator"
uiSuperUserNo="not $uiSuperUserYes"

uiLastChanceToCancel="Click \"Next\" to create this user."

uiTheEndMessage="The user \"%s\" has been added to the system."

# =======
# Globals
# =======
APPDIR=`dirname "$0"`
cd -P "$APPDIR"
APPDIR="`pwd`"

usersIcon="${APPDIR}/icons/users.xpm"
warningIcon="${APPDIR}/icons/warning.xpm"
errorIcon="${APPDIR}/icons/error.xpm"
tmpFile=/tmp/newuser.tmp.$$

dHeight=0		# height of dialog boxes (0 sets autosize)
dWidth=0		# width of dialog boxes (0 sets autosize)

# the following are used for "returning" values from functions
gStepNum=1	#0		# step in the wizard
gNewUsername=""	# stores new username
gAdditionalGroups=0		# 0 means 'add new user to "additional groups"'
gPlugdev=0				# 0 means 'add new user to plugdev group'
gMakeSuper=1			# 0 means 'make user a superuser'

# =========
# Functions
# =========

function checkName()
# checkName username
# if the format of "$username" is valid as a Debian username, return 0;
# otherwise return 1;
{
	local name="$1"
	local isFound=""
	
	# check number of arguments
	if [ -n "$2" ]; then
		echo -e "$0: too many parameters: (${2})" >&2
		return 1
	fi
	
	if [ -z "$1" ]; then
		echo -e "$0: not enough parameters" >&2
		return 1
	fi
	
	# search for forbidden characters (including spaces, etc.)
	# grep to find forbidden characters (i.e., NOT valid characters)
	isFound=$(echo "$name" | grep -E [^[:lower:][:digit:]_])
	# if $isFound is nonzero length, it means grep found at least one forbidden char
	if [ -n "$isFound" ]; then
		echo -e "found forbidden characters in \"$name\"" >&2
		return 1
	fi
	
	# check number of characters
	if [ ${#name} -lt 2 ]; then
		echo -e "not enough characters in \"$name\"" >&2
		return 1
	fi

	# if we made it here, there should be only one word consisting of
	# 'allowed' characters; we now have to see if the first char is a
	# lowercase letter.
	# grep if 1st char is a lower case letter
	isFound=$(echo "$name" | grep -E ^[[:lower:]])
	# isFound is now either a valid username or blank
	if [ -z $isFound ]; then
		echo -e "\"$name\" does not start with a lowercase letter." >&2
		return 1
	fi
	
	# we made it!
	return 0
}

function userExists()
# userExists username
# if "$username" already exists on this system, return 0;
# otherwise return 1;
{
	local isExist=""
	isExist=$(cat /etc/passwd | grep ^$1:)
	
	if [ -n "$isExist" ]; then
		return 0
	else
		return 1
	fi
}

function getUsername()
# getUsername promptString
# prompt user for new username using $promptString
# copy new username to global gNewUsername
# check for name validity and return 0 if name is valid; 1 otherwise
# This is a "wizard function"; it sets the global gStepNum to
# gStepNum+1 (next) or gStepNum-1 (previous).
{
	local tmpFile=/tmp/getUsername.tmp.$$
	local retval=-1
	local nameStr=""
	local tmpResult=""
	
	Xdialog --icon "$usersIcon" \
			--left \
			--wrap \
			--wizard \
			--title "$uiAddUser" \
			--inputbox "$1" $dHeight $dWidth "$gNewUsername" 2> $tmpFile
	retval=$?
	
	# get the filename
	nameStr=`cat $tmpFile`
	rm -f $tmpFile
	
	# depending on user response, exit or set to previous step and return
	if [ $retval -eq 1 ]; then		# cancel
		exit 1
	elif [ $retval -eq 3 ]; then	# previous
		let "gStepNum=$gStepNum-1"
		return
	fi

	# copy candidate new user name
	gNewUsername="$nameStr"
	
	# check for blank username
	if [ -z "$nameStr" ]; then
		# show a warning message
		Xdialog --icon "$warningIcon" \
			--left \
			--wrap \
			--title "$uiAddUser" \
			--msgbox "$uiNoBlankNames" $dHeight $dWidth
		# do NOT increment gStepNum so that we repeat this step
		return 1
	fi

	# check to see the new username is valid
	checkName "$gNewUsername"
	tmpResult=$?
	if [ $tmpResult -ne 0 ]; then
		# show an error message
		Xdialog --icon "$errorIcon" \
			--left \
			--wrap \
			--title "$uiAddUser" \
			--msgbox "`printf \"$uiInvalidName\" \"$gNewUsername\"`" $dHeight $dWidth
		# do NOT increment gStepNum so that we repeat this step
		return 1
	fi
	
	# check to see if name already exists
	userExists "$gNewUsername"
	tmpResult=$?
	if [ $tmpResult -eq 0 ]; then
		# show an error message
		Xdialog --icon "$errorIcon" \
			--left \
			--wrap \
			--title "$uiAddUser" \
			--msgbox "`printf \"$uiUserAlreadyExists\" \"$gNewUsername\"`" $dHeight $dWidth
		# do NOT increment gStepNum so that we repeat this step
		return 1
	fi	
	
	let "gStepNum=$gStepNum+1"
	return 0
}

function setOptions()
# Asks user if new user creation should --add_extra_groups and
# if user should be added to plugdev;
# sets gAdditionalGroups, gPlugdev and gMakeSuper accordingly
# This is a "wizard function"; it sets the global gStepNum to
# gStepNum+1 (next) or gStepNum-1 (previous).
{
	local tmpFile=/tmp/setOptions.tmp.$$
	local dialogValues=""	# used to get values from dialog box
	local isFound=1
	local additionalGroupsState="off"
	local plugdevState="off"
	local makesuperState="off"
	
	# preset checkbox states
	if [ $gAdditionalGroups -eq 0 ]; then
		additionalGroupsState="on"
	else
		additionalGroupsState="off"	
	fi
	
	if [ $gPlugdev -eq 0 ]; then
		plugdevState="on"
	else
		plugdevState="off"	
	fi
	
	if [ $gMakeSuper -eq 0 ]; then
		makesuperState="on"
	else
		makesuperState="off"	
	fi

	Xdialog --icon "$usersIcon" \
			--left \
			--wrap \
			--wizard \
			--help "$uiSetOptionsHelp" \
			--title "$uiAddUser" \
			--no-tags \
			--checklist "$uiSetOptionsInfo" $dHeight $dWidth 4 \
"additionalGroups"	"$uiSetOptionsExtraDesc" $additionalGroupsState \
"plugdev"			"$uiSetOptionsPlugdevDesc" $plugdevState \
"isSuperuser"		"$uiSetOptionsSuperUserDesc" $makesuperState \
        2> $tmpFile
	
	retval=$?
	# get the dialogValues
	dialogValues=`cat $tmpFile`
	rm -f $tmpFile

	# depending on user response, exit or set to previous step and return
	if [ $retval -eq 1 ]; then		# cancel
		exit 1
	elif [ $retval -eq 3 ]; then	# previous
		let "gStepNum=$gStepNum-1"
		return
	fi
	
	# if "additionalGroups" is in dialogValues, set gAdditionalGroups
	isFound=$(echo "$dialogValues" | grep additionalGroups)
	if [ -n "$isFound" ]; then
		gAdditionalGroups=0
	else
		gAdditionalGroups=1
	fi
	
	# if "plugdev" is in dialogValues, set gPlugdev
	isFound=$(echo "$dialogValues" | grep plugdev)
	if [ -n "$isFound" ]; then
		gPlugdev=0
	else
		gPlugdev=1
	fi
	
	# if "isSuperuser" is in dialogValues, set gMakeSuper
	isFound=$(echo "$dialogValues" | grep isSuperuser)
	if [ -n "$isFound" ]; then
		gMakeSuper=0
	else
		gMakeSuper=1
	fi
	
	let "gStepNum=$gStepNum+1"	
}

function makeSuper()
# Makes user in "$1" a superuser by adding user to group 'wheel'.
# Assumes $1 is a valid username that already exists on the system.
# Assimes group 'wheel' exists on the system and is configured in
# sudoers to facilitate rootiness.
{
	gpasswd -a $1 wheel
}

function makeUser()
# makeUser $1 $2 $3 $4
# $1 is <username>, a valid name is assumed!
# $2 is <isAddToExtraGroups>, 0 = yes
# $3 is <isAddToPlugdev>, 0 = yes
# $4 is <isMakeUserASuperuser>, 0 = yes
# This is a "wizard function"; it sets the global gStepNum to
# gStepNum+1 (next) or gStepNum-1 (previous).
{
	local newname=""
	local additionalGroups=1
	local addPlugdev=1
	local makeUserSuper=0
	local cmdStr1=""
	local cmdStr2=""
	local confirmStr=""
	local retval=""
	
	# get/check input params
	if [ -n "$1" ]; then
		newname="$1"
	else
		echo "ERROR: not enough parameters" >&2
		exit 1
	fi

	if [ -n "$2" ]; then
		additionalGroups=$2
	else
		additionalGroups=1
	fi

	if [ -n "$3" ]; then
		addPlugdev=$3
	else
		addPlugdev=1
	fi
	
	if [ -n "$4" ]; then
		makeUserSuper=$4
	else
		makeUserSuper=1
	fi

	# assemble command strings
	#cmdStr1="adduser"		# start building command to execute
	#confirmStr="`printf \"$uiConfirmUsername\" \"$newname\"`"

	#if [ $additionalGroups -eq 0 ]; then
		#cmdStr1="$cmdStr1 --add_extra_groups"
		#confirmStr="${confirmStr}\n  * ${uiExtraGroupsYes}"
	#else
		#confirmStr="${confirmStr}\n  * ${uiExtraGroupsNo}"
	#fi

	#cmdStr1="$cmdStr1 \"$newname\""
	
	confirmStr="`printf \"$uiConfirmUsername\" \"$newname\"`"
	
	if [ $additionalGroups -eq 0 ]; then
		# create user, add extra groups, add new user to netdev group as well
		# note: users need to be in netdev for NetworkManager to work.
		cmdStr1="adduser --add_extra_groups \"$newname\" ; adduser \"$newname\" netdev"
		confirmStr="${confirmStr}\n  * ${uiExtraGroupsYes}"
	else
		# just create a simple new user
		cmdStr1="adduser \"$newname\""
		confirmStr="${confirmStr}\n  * ${uiExtraGroupsNo}"
	fi

	if [ $addPlugdev -eq 0 ]; then
		cmdStr2="adduser \"$newname\" plugdev"
		confirmStr="${confirmStr}\n  * ${uiPlugdevYes}"
	else
		cmdStr2="echo"
		confirmStr="${confirmStr}\n  * ${uiPlugdevNo}"
	fi
		
	if [ $makeUserSuper -eq 0 ]; then
		confirmStr="${confirmStr}\n  * ${uiSuperUserYes}"
	else
		confirmStr="${confirmStr}\n  * ${uiSuperUserNo}"
	fi

	confirmStr="${confirmStr}\n\n${uiLastChanceToCancel}"
	
	# confirm stuff
	Xdialog --icon "$usersIcon" \
			--left \
			--wrap \
			--wizard \
			--title "$uiConfirmDetails" \
			--yesno "$confirmStr" $dHeight $dWidth
	retval=$?
	# depending on user response, exit or set to previous step and return
	if [ $retval -eq 1 ]; then		# cancel
		exit 1
	elif [ $retval -eq 3 ]; then	# previous
		let "gStepNum=$gStepNum-1"
		return
	fi

	# do it!
	urxvt -bg DarkRed -fg white -title "$uiAddUser" -n "$uiAddUser"\
		-e /bin/sh -c "$cmdStr1 ;\
		               $cmdStr2 ;\
		               echo ;\
		               read -p \"${uiPressKey2Exit}\""

	if [ $makeUserSuper -eq 0 ]; then
		makeSuper $newname
	fi
	
	skinny-reset-user -f -U $newname
	
	# move to next step
	let "gStepNum=$gStepNum+1"
}

function theEnd
# Lets user know they have finished the process
{
	Xdialog --icon "$usersIcon" \
			--left \
			--wrap \
			--title "$uiAddUser" \
			--msgbox "`printf \"$uiTheEndMessage\" \"$gNewUsername\"`" 0 0 
}

# ======
# "Main"
# ======

# Test for Xdialog
isXdialog=`which Xdialog`
if [ -z $isXdialog ] ; then
	echo "$uiXdialogNotFound"
	xmessage -center -buttons Ok "$uiXdialogNotFound"
	exit 1
fi

# go!
while true ; do
	case $gStepNum in
		0)
			# signal to calling ./ManageUsers.sh that it needs to show GUI
			# with "special" return value of 3
			exit 3
			;;
		1)
			getUsername "${uiNameRules}\n\n${uiEnterNewName}"
			continue
			;;
		2)
			setOptions
			continue
			;;
		3)
			makeUser "$gNewUsername" $gAdditionalGroups $gPlugdev $gMakeSuper
			continue
			;;
		4)
			theEnd
			break
			;;
		*)
			echo "ERROR: undefined program state" >&2
			exit 1
	esac
done
