#!/bin/bash

#Mithat Konar - 2009-08-03

# GUI wizard wrapper for Debian's deluser.

# ==========
# UI strings
# ==========
uiXdialogNotFound="Xdialog was not found.
Please install Xdialog to use this script."
uiSedNotFound="sed was not found.
Please install sed to use this script."

uiDeleteUser="Delete user"
uiOk="Ok"
uiCancel="Cancel"
uiConfirmDetails="Confirm delete"
uiPressKey2Exit="Press ENTER to continue . . ."

uiDeleteUserIntro="Select the username of the account to delete:"
uiNoBlankNames="The username cannot be blank."
uiInvalidName="\"%s\" is not a valid username."
uiUserDoesntExist="The user \"%s\" does not exist on this system."

uiDeleteHomeDesc="Also delete the user's home directory and all files in it"
uiDeleteUserHelp="\
If you don't see the name of the user you want to delete in the\n\
list, the user is not a normal user. You will need to use the\n\
command-line \"userdel\" tool instead of this one to delete the user."

uiConfirmUsername="I am about to delete the user \"%s\""
uiDeleteHomeYes="and erase their home directory and all files in it."
uiDeleteHomeNo="but keep their home directory and all files in it."
uiCantBeUndone="THIS CANNOT BE UNDONE!"
uiDeleteUserConfirm="Delete user \"%s\""

uiTheEndMessage="The user \"%s\" has been removed from the system."

# =======
# Globals
# =======
APPDIR=`dirname "$0"`
cd -P "$APPDIR"
APPDIR="`pwd`"

usersIcon="${APPDIR}/icons/users.xpm"
warningIcon="${APPDIR}/icons/warning.xpm"
errorIcon="${APPDIR}/icons/error.xpm"
tmpFile=/tmp/removeuser.tmp.$$

# the following are used for "returning" values from functions
gStepNum=1		# step in the wizard
gUsername=""	# stores new username
gDeleteHome=0	# 0 means 'delete user's home directory as well'

# =========
# Functions
# =========

function checkName()
# checkName username
# if the format of "$username" is valid as a Debian username, return 0;
# otherwise return 1;
{
	local name="$1"
	local isFound=""
	
	# check number of arguments
	if [ -n "$2" ]; then
		echo -e "$0: too many parameters: (${2})" >&2
		return 1
	fi
	
	if [ -z "$1" ]; then
		echo -e "$0: not enough parameters" >&2
		return 1
	fi
	
	# search for forbidden characters (including spaces, etc.)
	# grep to find forbidden characters (i.e., NOT valid characters)
	isFound=$(echo "$name" | grep -E [^[:lower:][:digit:]_])
	# if $isFound is nonzero length, it means grep found at least one forbidden char
	if [ -n "$isFound" ]; then
		echo -e "found forbidden characters in \"$name\"" >&2
		return 1
	fi
	
	# check number of characters
	if [ ${#name} -lt 2 ]; then
		echo -e "not enough characters in \"$name\"" >&2
		return 1
	fi

	# if we made it here, there should be only one word consisting of
	# 'allowed' characters; we now have to see if the first char is a
	# lowercase letter.
	# grep if 1st char is a lower case letter
	isFound=$(echo "$name" | grep -E ^[[:lower:]])
	# isFound is now either a valid username or blank
	if [ -z $isFound ]; then
		echo -e "\"$name\" does not start with a lowercase letter." >&2
		return 1
	fi
	
	# we made it!
	return 0
}

function userExists()
# userExists username
# if "$username" already exists on this system, return 0;
# otherwise return 1;
{
	local isExist=""
	isExist=$(cat /etc/passwd | grep ^$1:)
	
	if [ -n "$isExist" ]; then
		return 0
	else
		return 1
	fi
}

function getUsername()
# getUsername
# prompt for username to delete and whether home dir should be deleted.
# copy username to global gUsername, copy home dir delete state to gDeleteHome
# check for name validity and return 0 if name is valid; 1 otherwise
# This is a "wizard function"; it sets the global gStepNum to
# gStepNum+1 (next) or gStepNum-1 (previous).
{
	local tmpFile=/tmp/getUsername.tmp.$$
	local retval=-1
	local nameStr=""
	local isDelHomeDir=""	# used to set UI checkbox and get its state
	local tmpResult=""
	
	if [ $gDeleteHome -eq 0 ]; then
		isDelHomeDir="on"
	else
		isDelHomeDir="off"
	fi
	
	# make a list of users in $tmpFile
	cat /etc/passwd | grep /home/ | grep -v /bin/false | cut -d":" -f1 > $tmpFile
	sort -n --output=$tmpFile $tmpFile		# sort the list

	local buttonsString=""
	while read line
	do
	{
		buttonsString="$buttonsString $line $line \"off\""
	}
	done< $tmpFile
	rm -f $tmpFile							# we're done with tmpFile

	Xdialog --icon "$usersIcon" \
			--left \
			--wrap \
			--wizard \
			--help "$uiDeleteUserHelp" \
			--title "$uiDeleteUser" \
			--check "$uiDeleteHomeDesc" "$isDelHomeDir" \
			--no-tags \
			--radiolist "$uiDeleteUserIntro" 0 0 5 $buttonsString 2> $tmpFile
	retval=$?	
	
	nameStr=`sed -n '1p' "$tmpFile"`		# get the filename in $tmpFile
	isDelHomeDir=`sed -n '2p' "$tmpFile"`	# get the home dir state in $tmpFile
	rm -f $tmpFile							# we're done with tmpFile
	
	# depending on user response, exit or set to previous step and return
	if [ $retval -eq 1 ]; then		# cancel
		exit 1
	elif [ $retval -eq 3 ]; then	# previous
		let "gStepNum=$gStepNum-1"
		return
	fi

	# set globals used for function output
	gUsername="$nameStr"						# set new user name
	if [ "$isDelHomeDir" = "checked" ]; then	# set 'delete home dir' state
		gDeleteHome=0
	else
		gDeleteHome=1
	fi

	# The following checks should not be needed since the selections
	# above should never produce unvalid output. But just in case...
	# check for blank username
	if [ -z "$gUsername" ]; then
		# show a warning message
		Xdialog --icon "$warningIcon" \
			--left \
			--wrap \
			--title "$uiDeleteUser" \
			--msgbox "$uiNoBlankNames" 0 0
		# do NOT increment gStepNum so that we repeat this step
		return 1
	fi

	# check to see the new username is valid
	checkName "$gUsername"
	tmpResult=$?
	if [ $tmpResult -ne 0 ]; then
		# show an error message
		Xdialog --icon "$errorIcon" \
			--left \
			--wrap \
			--title "$uiDeleteUser" \
			--msgbox "`printf \"$uiInvalidName\" \"$gUsername\"`" 0 0
		# do NOT increment gStepNum so that we repeat this step
		return 1
	fi
	
	# check to see if name already exists
	userExists "$gUsername"
	tmpResult=$?
	if [ $tmpResult -ne 0 ]; then
		# show an error message
		Xdialog --icon "$errorIcon" \
			--left \
			--wrap \
			--title "$uiDeleteUser" \
			--msgbox "`printf \"$uiUserDoesntExist\" \"$gUsername\"`" 0 0
		# do NOT increment gStepNum so that we repeat this step
		return 1
	fi	
	
	let "gStepNum=$gStepNum+1"
	return 0
}

function removeUser()
# removeUser $1 $2
# $1 is <username>, a valid name is assumed!
# $2 is <isRemoveHome>, remove user's home directory or not; 0 = yes
# Asks to confirm actions
# This is a "wizard function"; it sets the global gStepNum to
# gStepNum+1 (next) or gStepNum-1 (previous).
{
	local u_name=""
	local isRemoveHome=1
	local cmdStr1=""
	local cmdStr2=""
	local confirmStr=""
	local retval=""
	
	# get/check input params
	if [ -n "$1" ]; then
		u_name="$1"
	else
		echo "ERROR: not enough parameters" >&2
		exit 1
	fi

	if [ -n $2 ]; then
		isRemoveHome=$2
	else
		isRemoveHome=1
	fi

	# assemble command strings
	cmdStr1="deluser"		# start building command to execute
	confirmStr="`printf \"$uiConfirmUsername\" \"$u_name\"`"

	if [ $isRemoveHome -eq 0 ]; then
		cmdStr1="$cmdStr1 --remove-home"
		confirmStr="${confirmStr}\n${uiDeleteHomeYes}"
	else
		confirmStr="${confirmStr}\n${uiDeleteHomeNo}"
	fi

	cmdStr1="$cmdStr1 $u_name"

	## confirm stuff (full wizard version)
	#Xdialog --icon "$usersIcon" \
			#--left \
			#--wrap \
			#--wizard \
			#--title "$uiConfirmDetails" \
			#--yesno "$confirmStr" 0 0
	#retval=$?

	# confirm stuff (non-wizard version)
	Xdialog --icon "$usersIcon" \
			--left \
			--wrap \
			--title "$uiConfirmDetails" \
			--ok-label "`printf \"$uiDeleteUserConfirm\" \"$u_name\"`" \
			--cancel-label "$uiCancel" \
			--default-no \
			--yesno "$confirmStr\n\n$uiCantBeUndone" 0 0
	retval=$?
	
	# depending on user response, exit or set to previous step and return
	if [ $retval -eq 1 ]; then		# cancel
		exit 1
	elif [ $retval -eq 3 ]; then	# previous
		let "gStepNum=$gStepNum-1"
		return
	fi
	
	# do it!
	#echo "DEBUG: cmdStr1 = $cmdStr1"
	urxvt -bg DarkRed -fg white -title "$uiDeleteUser" -n "$uiDeleteUser"\
		-e /bin/sh -c "$cmdStr1 ;\
		               echo ;\
		               read -p \"${uiPressKey2Exit}\""

	# remove sudo time-lock directory
	local sudoDirPath="/var/run/sudo/$u_name"
	rm -rf "$sudoDirPath"		# remove pre-existing folder/files (if any)
	
	# move to next step
	let "gStepNum=$gStepNum+1"
}

function theEnd
# Lets user know they have finished the process
{
	Xdialog --icon "$usersIcon" \
			--left \
			--wrap \
			--title "$uiDeleteUser" \
			--msgbox "`printf \"$uiTheEndMessage\" \"$gUsername\"`" 0 0 
}

# ======
# "Main"
# ======

# Test for Xdialog
isExist=`which Xdialog`
if [ -z $isExist ] ; then
	echo "${0}: $uiXdialogNotFound"
	xmessage -center -buttons Ok "$uiXdialogNotFound"
	exit 1
fi
# Test for sed
isExist=`which sed`
if [ -z $isExist ] ; then
	#echo "${0}: $uiSedNotFound"
	xmessage -center -buttons Ok "$uiSedNotFound"
	exit 1
fi

# go!
while true ; do
	case $gStepNum in
		0)
			# signal to calling ./ManageUsers.sh that it needs to show GUI
			# with "special" return value of 3
			exit 3
			;;
		1)
			getUsername
			continue
			;;
		2)
			removeUser "$gUsername" $gDeleteHome
			continue;
			;;
		3)
			theEnd
			break
			;;
		*)
			echo "ERROR: undefined program state" >&2
			exit 1
	esac
done
