#!/bin/bash

# Mithat Konar - 2009-07-27

# GUI wizard for resetting a user's SkinnyDebbie environment (run as root).

# requires Xdialog, skinny-utils

# ==========
# UI strings
# ==========
uiResetSkinny="Reset SkinnyDebbie environment"
uiOk="Ok"
uiCancel="Cancel"
#uiConfirmDetails="Confirm change"
uiPressKey2Exit="Press ENTER to continue . . ."

uiResetSkinnyIntro="\
Select the name of the user whose environment you want to reset:"
uiResetSkinnyHelp="\
If a user's SkinnyDebbie environment gets corrupted or otherwise\n\
becomes unusable, you can reset it to the \"factory\" defaults here.\n\n\
You should be aware that this will cause the loss of most custom\n\
desktop settings and some application settings.\n\n\
Use with care."

#uiConfirmUsername="I am about to change %s's password."
#uiCantBeUndone="This cannot be undone."
#uiChangeUserConfirm="Change password for \"%s\""

uiTheEndMessage="Finished reseting %s's environment."

# =======
# Globals
# =======
APPDIR=`dirname "$0"`
cd -P "$APPDIR"
APPDIR="`pwd`"

usersIcon="${APPDIR}/icons/users.xpm"
warningIcon="${APPDIR}/icons/warning.xpm"
errorIcon="${APPDIR}/icons/error.xpm"
tmpFile=/tmp/reset-skinny.tmp.$$

# the following are used for "returning" values from functions
gStepNum=1		# step in the wizard
gUsername=""	# stores new username

# =========
# Functions
# =========

function getUsername()
# getUsername
# prompt for username to change passwd.
# copy username to global gUsername.
# check for name validity and return 0 if name is valid; 1 otherwise
# This is a "wizard function"; it sets the global gStepNum to
# gStepNum+1 (next) or gStepNum-1 (previous).
{
	local tmpFile=/tmp/getUsername.tmp.$$
	local retval=-1
	local nameStr=""
	local tmpResult=""
	
	# make a list of users in $tmpFile
	cat /etc/passwd | grep /home/ | grep -v /bin/false | cut -d":" -f1 > $tmpFile
	sort -n --output=$tmpFile $tmpFile		# sort the list

	local buttonsString=""
	while read line
	do
	{
		buttonsString="$buttonsString $line $line \"off\""
	}
	done< $tmpFile
	rm -f $tmpFile							# we're done with tmpFile

	Xdialog --icon "$usersIcon" \
			--left \
			--wrap \
			--wizard \
			--help "$uiResetSkinnyHelp" \
			--title "$uiResetSkinny" \
			--no-tags \
			--radiolist "$uiResetSkinnyIntro" 0 0 5 $buttonsString 2> $tmpFile
	retval=$?	
	
	nameStr=`sed -n '1p' "$tmpFile"`		# get the filename in $tmpFile
	rm -f $tmpFile							# we're done with tmpFile
	
	# depending on user response, exit or set to previous step and return
	if [ $retval -eq 1 ]; then		# cancel
		exit 1
	elif [ $retval -eq 3 ]; then	# previous
		let "gStepNum=$gStepNum-1"
		return
	fi

	# set globals used for function output
	gUsername="$nameStr"						# set new user name
		
	let "gStepNum=$gStepNum+1"
	return 0
}

function resetEnv()
# resetEnv $1
# $1 is <username>, a valid name is assumed!
# Resets username's SkinnyDebbie environment
# Asks to confirm actions
# This is a "wizard function"; it sets the global gStepNum to
# gStepNum+1 (next) or gStepNum-1 (previous).
{
	local u_name=""
	local cmdStr1=""
	#local confirmStr=""
	local retval=""
	
	# get/check input params
	if [ -n "$1" ]; then
		u_name="$1"
	else
		echo "ERROR: not enough parameters" >&2
		exit 1
	fi

	# assemble command strings
	cmdStr1="skinny-reset-user -U $u_name"		# start building command to execute
	
	# do it!
	urxvt -bg DarkRed -fg white -title "$uiResetSkinny" -n "$uiResetSkinny"\
		-e /bin/sh -c "$cmdStr1 ;\
		               echo ;\
		               read -p \"${uiPressKey2Exit}\""

	# move to next step
	let "gStepNum=$gStepNum+1"
}

function theEnd
# Lets user know they have finished the process
{
	Xdialog --icon "$usersIcon" \
			--left \
			--wrap \
			--title "$uiResetSkinny" \
			--msgbox "`printf \"$uiTheEndMessage\" \"$gUsername\"`" 0 0 
}

# ======
# "Main"
# ======

while true ; do
	case $gStepNum in
		0)
			# signal to calling script that it needs to show GUI
			# by using "special" return value of 3
			exit 3
			;;
		1)
			getUsername
			continue
			;;
		2)
			resetEnv "$gUsername"
			continue;
			;;
		3)
			theEnd
			break
			;;
		*)
			echo "ERROR: undefined program state" >&2
			exit 1
	esac
done
